@@ -8,12 +8,15 @@ from allauth.socialaccount.models import SocialAccount, SocialLogin
 def sociallogin_factory(user_factory):
     def factory(
         email=None,
+        username=None,
         with_email=True,
         provider="unittest-server",
         uid="123",
         email_verified=True,
     ):
-        user = user_factory(email=email, commit=False, with_email=with_email)
+        user = user_factory(
+            username=username, email=email, commit=False, with_email=with_email
+        )
         account = SocialAccount(provider=provider, uid=uid)
         sociallogin = SocialLogin(user=user, account=account)
         if with_email:
