@@ -1,12 +1,12 @@
 from django.conf import settings
 from django.contrib.auth import authenticate, get_user_model
 from django.contrib.sites.shortcuts import get_current_site
-from django.core.exceptions import PermissionDenied
+from django.core.exceptions import ImproperlyConfigured, PermissionDenied
 from django.db import models
-from django.utils.crypto import get_random_string
 from django.utils.translation import gettext_lazy as _
 
 import allauth.app_settings
+from allauth import app_settings as allauth_settings
 from allauth.account.models import EmailAddress
 from allauth.account.utils import (
     filter_users_by_email,
@@ -15,12 +15,19 @@ from allauth.account.utils import (
 )
 from allauth.core import context
 from allauth.socialaccount import signals
+from allauth.socialaccount.internal import statekit
 
 from ..utils import get_request_param
 from . import app_settings, providers
 from .adapter import get_adapter
 
 
+if not allauth_settings.SOCIALACCOUNT_ENABLED:
+    raise ImproperlyConfigured(
+        "allauth.socialaccount not installed, yet its models are imported."
+    )
+
+
 class SocialAppManager(models.Manager):
     def on_site(self, request):
         if allauth.app_settings.SITES_ENABLED:
@@ -36,7 +43,6 @@ class SocialApp(models.Model):
     provider = models.CharField(
         verbose_name=_("provider"),
         max_length=30,
-        choices=providers.registry.as_choices(),
     )
     # For providers that support subproviders, such as OpenID Connect and SAML,
     # this ID identifies that instance. SocialAccount's originating from app
@@ -223,6 +229,10 @@ class SocialLogin(object):
             },
         )
 
+    @property
+    def is_headless(self):
+        return bool(self.state.get("headless"))
+
     def serialize(self):
         serialize_instance = get_adapter().serialize_instance
         ret = dict(
@@ -274,19 +284,34 @@ class SocialLogin(object):
         else:
             setup_user_email(request, user, self.email_addresses)
 
+    @property
+    def user(self):
+        return self._user
+
+    @user.setter
+    def user(self, v):
+        self._user = v
+        self._is_existing = None
+
     @property
     def is_existing(self):
         """When `False`, this social login represents a temporary account, not
         yet backed by a database record.
         """
+        if self._is_existing is not None:
+            return self._is_existing
         if self.user.pk is None:
-            return False
-        return get_user_model().objects.filter(pk=self.user.pk).exists()
+            ret = False
+        else:
+            ret = get_user_model().objects.filter(pk=self.user.pk).exists()
+        self._is_existing = ret
+        return ret
 
     def lookup(self):
         """Look up the existing local user account to which this social login
         points, if any.
         """
+        self._did_authenticate_by_email = False
         if not self._lookup_by_socialaccount():
             self._lookup_by_email()
 
@@ -341,16 +366,23 @@ class SocialLogin(object):
             users = filter_users_by_email(email, prefer_verified=True)
             if users:
                 self.user = users[0]
-                if app_settings.EMAIL_AUTHENTICATION_AUTO_CONNECT:
-                    self.connect(context.request, self.user)
+                self._did_authenticate_by_email = True
                 return
 
+    def _accept_login(self):
+        if self._did_authenticate_by_email:
+            if app_settings.EMAIL_AUTHENTICATION_AUTO_CONNECT:
+                self.connect(context.request, self.user)
+
     def get_redirect_url(self, request):
         url = self.state.get("next")
         return url
 
     @classmethod
     def state_from_request(cls, request):
+        """
+        TODO: Deprecated! To be integrated with provider.redirect()
+        """
         state = {}
         next_url = get_next_redirect_url(request)
         if next_url:
@@ -361,24 +393,22 @@ class SocialLogin(object):
         return state
 
     @classmethod
-    def stash_state(cls, request):
-        state = cls.state_from_request(request)
-        verifier = get_random_string(16)
-        request.session["socialaccount_state"] = (state, verifier)
-        return verifier
+    def stash_state(cls, request, state=None):
+        if state is None:
+            # Only for providers that don't support redirect() yet.
+            state = cls.state_from_request(request)
+        return statekit.stash_state(request, state)
 
     @classmethod
     def unstash_state(cls, request):
-        if "socialaccount_state" not in request.session:
+        state = statekit.unstash_last_state(request)
+        if state is None:
             raise PermissionDenied()
-        state, verifier = request.session.pop("socialaccount_state")
         return state
 
     @classmethod
     def verify_and_unstash_state(cls, request, verifier):
-        if "socialaccount_state" not in request.session:
-            raise PermissionDenied()
-        state, verifier2 = request.session.pop("socialaccount_state")
-        if verifier != verifier2:
+        state = statekit.unstash_state(request, verifier)
+        if state is None:
             raise PermissionDenied()
         return state
