@@ -1,7 +1,7 @@
 from unittest.mock import Mock, patch
 from urllib.parse import parse_qs, urlparse
 
-from django.urls import reverse
+from django.urls import reverse, reverse_lazy
 from django.utils.http import urlencode
 
 import pytest
@@ -9,17 +9,18 @@ from pytest_django.asserts import assertTemplateUsed
 
 from allauth.account.models import EmailAddress
 from allauth.socialaccount.adapter import get_adapter
+from allauth.socialaccount.internal import statekit
 from allauth.socialaccount.models import SocialAccount
 from allauth.socialaccount.providers.saml.utils import build_saml_config
 
 
 @pytest.mark.parametrize(
-    "is_connect,relay_state, expected_url",
+    "is_connect,state_kwargs,relay_state, expected_url",
     [
-        (False, None, "/accounts/profile/"),
-        (False, "/foo", "/foo"),
-        (True, "process=connect", "/social/connections/"),
-        (True, "process=connect&next=/conn", "/conn"),
+        (False, None, None, "/accounts/profile/"),
+        (False, None, "/foo", "/foo"),
+        (True, {"process": "connect"}, None, reverse_lazy("socialaccount_connections")),
+        (True, {"process": "connect", "next_url": "/conn"}, None, "/conn"),
     ],
 )
 def test_acs(
@@ -31,6 +32,8 @@ def test_acs(
     mocked_signature_validation,
     expected_url,
     relay_state,
+    state_kwargs,
+    sociallogin_setup_state,
 ):
     if is_connect:
         client = request.getfixturevalue("auth_client")
@@ -39,6 +42,11 @@ def test_acs(
         client = request.getfixturevalue("client")
         user = None
 
+    if state_kwargs:
+        assert not relay_state
+        state_id = sociallogin_setup_state(client, **state_kwargs)
+        relay_state = urlencode({"state": state_id})
+
     data = {"SAMLResponse": acs_saml_response}
     if relay_state is not None:
         data["RelayState"] = relay_state
@@ -86,27 +94,19 @@ def test_login_on_get(client, db, saml_settings):
     assertTemplateUsed(resp, "socialaccount/login.html")
 
 
-@pytest.mark.parametrize(
-    "query,expected_relay_state",
-    [
-        ("", None),
-        ("?process=connect", "process=connect"),
-        ("?process=connect&next=/foo", "process=connect&next=%2Ffoo"),
-        ("?next=/bar", "next=%2Fbar"),
-    ],
-)
-def test_login(client, db, saml_settings, query, expected_relay_state):
+def test_login(client, db, saml_settings):
     resp = client.post(
-        reverse("saml_login", kwargs={"organization_slug": "org"}) + query
+        reverse("saml_login", kwargs={"organization_slug": "org"})
+        + "?process=connect&next=/foo"
     )
     assert resp.status_code == 302
     location = resp["location"]
     assert location.startswith("https://dev-123.us.auth0.com/samlp/456?SAMLRequest=")
     resp_query = parse_qs(urlparse(location).query)
-    if expected_relay_state is None:
-        assert "RelayState" not in resp_query
-    else:
-        assert resp_query.get("RelayState")[0] == expected_relay_state
+    relay_state = resp_query.get("RelayState")[0]
+    state_id = parse_qs(relay_state)["state"][0]
+    state = client.session[statekit.STATES_SESSION_KEY][state_id][0]
+    assert state == {"process": "connect", "data": None, "next": "/foo"}
 
 
 def test_metadata(
@@ -166,6 +166,13 @@ def test_build_saml_config_without_metadata_url(rf, provider_config):
                 "metadata_url": "https://idp.org/sso/",
             }
         },
+        {
+            "idp": {
+                "entity_id": "dummy",
+                "metadata_url": "https://idp.org/sso/",
+            },
+            "sp": {"entity_id": "dummy-sp-entity-id"},
+        },
     ],
 )
 def test_build_saml_config(rf, provider_config):
@@ -187,6 +194,12 @@ def test_build_saml_config(rf, provider_config):
     assert config["idp"]["x509cert"] == "cert"
     assert config["idp"]["singleSignOnService"] == {"url": "https://idp.org/sso/"}
     assert config["idp"]["singleLogoutService"] == {"url": "https://idp.saml.org/slo/"}
+    metadata_url = request.build_absolute_uri(reverse("saml_metadata", args=["org"]))
+    sp_entity_id = provider_config.get("sp", {}).get("entity_id")
+    if sp_entity_id:
+        assert config["sp"]["entityId"] == sp_entity_id
+    else:
+        assert config["sp"]["entityId"] == metadata_url
 
 
 @pytest.mark.parametrize(
